@@ -19,7 +19,10 @@
 #include "config.h"
 #define _GNU_SOURCE /* for dup3 */
 
-#include "terminal-regex.h"
+#ifdef WITH_PCRE2
+#include "terminal-pcre2.h"
+#endif
+
 #include "terminal-screen.h"
 
 #include <errno.h>
@@ -72,7 +75,7 @@ typedef struct {
 typedef struct
 {
   int tag;
-  TerminalURLFlavour flavor;
+  TerminalURLFlavor flavor;
 } TagData;
 
 struct _TerminalScreenPrivate
@@ -149,6 +152,9 @@ static void terminal_screen_icon_title_changed        (VteTerminal *vte_terminal
 
 static void update_color_scheme                      (TerminalScreen *screen);
 
+static void terminal_screen_check_extra (TerminalScreen *screen,
+                                         GdkEvent       *event,
+                                         char           **number_info);
 static char* terminal_screen_check_match       (TerminalScreen            *screen,
                                                 GdkEvent                  *event,
                                                 int                  *flavor);
@@ -159,23 +165,47 @@ static void terminal_screen_set_override_command (TerminalScreen  *screen,
 
 static guint signals[LAST_SIGNAL];
 
+#define USERCHARS "-[:alnum:]"
+#define USERCHARS_CLASS "[" USERCHARS "]"
+#define PASSCHARS_CLASS "[-[:alnum:]\\Q,?;.:/!%$^*&~\"#'\\E]"
+#define HOSTCHARS_CLASS "[-[:alnum:]]"
+#define HOST HOSTCHARS_CLASS "+(\\." HOSTCHARS_CLASS "+)*"
+#define PORT "(?:\\:[[:digit:]]{1,5})?"
+#define PATHCHARS_CLASS "[-[:alnum:]\\Q_$.+!*,:;@&=?/~#%\\E]"
+#define PATHTERM_CLASS "[^\\Q]'.:}>) \t\r\n,\"\\E]"
+#define SCHEME "(?:news:|telnet:|nntp:|file:\\/|https?:|ftps?:|sftp:|webcal:)"
+#define USERPASS USERCHARS_CLASS "+(?:" PASSCHARS_CLASS "+)?"
+#define URLPATH   "(?:(/"PATHCHARS_CLASS"+(?:[(]"PATHCHARS_CLASS"*[)])*"PATHCHARS_CLASS"*)*"PATHTERM_CLASS")?"
+
 typedef struct {
   const char *pattern;
-  TerminalURLFlavour flavor;
+  TerminalURLFlavor flavor;
+  gboolean caseless;
 } TerminalRegexPattern;
 
 static const TerminalRegexPattern url_regex_patterns[] = {
-  { REGEX_URL_AS_IS, FLAVOR_AS_IS },
-  { REGEX_URL_HTTP,  FLAVOR_DEFAULT_TO_HTTP },
-  { REGEX_URL_FILE,  FLAVOR_AS_IS },
-  { REGEX_URL_VOIP,  FLAVOR_VOIP_CALL },
-  { REGEX_EMAIL,     FLAVOR_EMAIL },
-  { REGEX_NEWS_MAN,  FLAVOR_AS_IS },
+  { SCHEME "//(?:" USERPASS "\\@)?" HOST PORT URLPATH, FLAVOR_AS_IS, TRUE },
+  { "(?:www|ftp)" HOSTCHARS_CLASS "*\\." HOST PORT URLPATH , FLAVOR_DEFAULT_TO_HTTP, TRUE  },
+  { "(?:callto:|h323:|sip:)" USERCHARS_CLASS "[" USERCHARS ".]*(?:" PORT "/[a-z0-9]+)?\\@" HOST, FLAVOR_VOIP_CALL, TRUE },
+  { "(?:mailto:)?" USERCHARS_CLASS "[" USERCHARS ".]*\\@" HOSTCHARS_CLASS "+\\." HOST, FLAVOR_EMAIL, TRUE },
+  { "(?:news:|man:|info:)[-[:alnum:]\\Q^_{|}~!\"#$%&'()*+,./;:=?`\\E]+", FLAVOR_AS_IS, TRUE },
 };
 
+static const TerminalRegexPattern extra_regex_patterns[] = {
+  { "(0[Xx][[:xdigit:]]+|[[:digit:]]+)", FLAVOR_NUMBER, FALSE },
+};
+
+#ifdef WITH_PCRE2
+static VteRegex **url_regexes;
+static VteRegex **extra_regexes;
+#else
 static GRegex **url_regexes;
-static TerminalURLFlavour *url_regex_flavors;
+static GRegex **extra_regexes;
+#endif
+static TerminalURLFlavor *url_regex_flavors;
+static TerminalURLFlavor *extra_regex_flavors;
 static guint n_url_regexes;
+static guint n_extra_regexes;
 
 /* See bug #697024 */
 #ifndef __linux__
@@ -201,6 +231,53 @@ free_tag_data (TagData *tagdata)
   g_slice_free (TagData, tagdata);
 }
 
+static void
+precompile_regexes (const TerminalRegexPattern *regex_patterns,
+                    guint n_regexes,
+#ifdef WITH_PCRE2
+                    VteRegex ***regexes,
+#else
+                    GRegex ***regexes,
+#endif
+                    TerminalURLFlavor **regex_flavors)
+{
+  guint i;
+
+#ifdef WITH_PCRE2
+  *regexes = g_new0 (VteRegex*, n_regexes);
+#else
+  *regexes = g_new0 (GRegex*, n_regexes);
+#endif
+  *regex_flavors = g_new0 (TerminalURLFlavor, n_regexes);
+
+  for (i = 0; i < n_regexes; ++i)
+    {
+      GError *error = NULL;
+
+#ifdef WITH_PCRE2
+      (*regexes)[i] = vte_regex_new (regex_patterns[i].pattern, -1,
+                                     PCRE2_UTF | PCRE2_NO_UTF_CHECK |
+                                     (regex_patterns[i].caseless ? PCRE2_CASELESS : 0),
+                                     &error);
+      g_assert_no_error (error);
+
+      if (!vte_regex_jit ((*regexes)[i], PCRE2_JIT_COMPLETE, &error) ||
+          !vte_regex_jit ((*regexes)[i], PCRE2_JIT_PARTIAL_SOFT, &error)) {
+        g_printerr ("Failed to JIT regex '%s': %s\n", regex_patterns[i].pattern, error->message);
+        g_clear_error (&error);
+      }
+#else
+      (*regexes)[i] = g_regex_new (regex_patterns[i].pattern,
+                                   G_REGEX_OPTIMIZE |
+                                   (regex_patterns[i].caseless ? G_REGEX_CASELESS : 0),
+                                   0, &error);
+      g_assert_no_error (error);
+#endif
+
+      (*regex_flavors)[i] = regex_patterns[i].flavor;
+    }
+}
+
 static void
 terminal_screen_class_enable_menu_bar_accel_notify_cb (GSettings *settings,
                                                        const char *key,
@@ -321,7 +398,11 @@ terminal_screen_init (TerminalScreen *screen)
 
       tag_data = g_slice_new (TagData);
       tag_data->flavor = url_regex_flavors[i];
+#ifdef WITH_PCRE2
+      tag_data->tag = vte_terminal_match_add_regex (terminal, url_regexes[i], 0);
+#else
       tag_data->tag = vte_terminal_match_add_gregex (terminal, url_regexes[i], 0);
+#endif
       vte_terminal_match_set_cursor_type (terminal, tag_data->tag, URL_MATCH_CURSOR);
 
       priv->match_tags = g_slist_prepend (priv->match_tags, tag_data);
@@ -429,7 +510,6 @@ terminal_screen_class_init (TerminalScreenClass *klass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
   VteTerminalClass *terminal_class = VTE_TERMINAL_CLASS (klass);
   GSettings *settings;
-  guint i;
 
   object_class->constructed = terminal_screen_constructed;
   object_class->dispose = terminal_screen_dispose;
@@ -523,22 +603,10 @@ terminal_screen_class_init (TerminalScreenClass *klass)
 
   g_type_class_add_private (object_class, sizeof (TerminalScreenPrivate));
 
-  /* Precompile the regexes */
   n_url_regexes = G_N_ELEMENTS (url_regex_patterns);
-  url_regexes = g_new0 (GRegex*, n_url_regexes);
-  url_regex_flavors = g_new0 (TerminalURLFlavour, n_url_regexes);
-
-  for (i = 0; i < n_url_regexes; ++i)
-    {
-      GError *error = NULL;
-
-      url_regexes[i] = g_regex_new (url_regex_patterns[i].pattern,
-                                    G_REGEX_OPTIMIZE,
-                                    0, &error);
-      g_assert_no_error (error);
-
-      url_regex_flavors[i] = url_regex_patterns[i].flavor;
-    }
+  precompile_regexes (url_regex_patterns, n_url_regexes, &url_regexes, &url_regex_flavors);
+  n_extra_regexes = G_N_ELEMENTS (extra_regex_patterns);
+  precompile_regexes (extra_regex_patterns, n_extra_regexes, &extra_regexes, &extra_regex_flavors);
 
   /* This fixes bug #329827 */
   settings = terminal_app_get_global_settings (terminal_app_get ());
@@ -1389,7 +1457,8 @@ terminal_screen_popup_info_unref (TerminalScreenPopupInfo *info)
 
   g_object_unref (info->screen);
   g_weak_ref_clear (&info->window_weak_ref);
-  g_free (info->string);
+  g_free (info->url);
+  g_free (info->number_info);
   g_slice_free (TerminalScreenPopupInfo, info);
 }
 
@@ -1426,8 +1495,9 @@ terminal_screen_popup_menu (GtkWidget *widget)
 static void
 terminal_screen_do_popup (TerminalScreen *screen,
                           GdkEventButton *event,
-                          char *matched_string,
-                          int matched_flavor)
+                          char *url,
+                          int url_flavor,
+                          char *number_info)
 {
   TerminalScreenPopupInfo *info;
 
@@ -1435,8 +1505,9 @@ terminal_screen_do_popup (TerminalScreen *screen,
   info->button = event->button;
   info->state = event->state & gtk_accelerator_get_default_mod_mask ();
   info->timestamp = event->time;
-  info->string = matched_string; /* adopted */
-  info->flavour = matched_flavor;
+  info->url = url; /* adopted */
+  info->url_flavor = url_flavor;
+  info->number_info = number_info; /* adopted */
 
   g_signal_emit (screen, signals[SHOW_POPUP_MENU], 0, info);
   terminal_screen_popup_info_unref (info);
@@ -1449,23 +1520,25 @@ terminal_screen_button_press (GtkWidget      *widget,
   TerminalScreen *screen = TERMINAL_SCREEN (widget);
   gboolean (* button_press_event) (GtkWidget*, GdkEventButton*) =
     GTK_WIDGET_CLASS (terminal_screen_parent_class)->button_press_event;
-  gs_free char *matched_string = NULL;
-  int matched_flavor = 0;
+  gs_free char *url = NULL;
+  int url_flavor = 0;
+  gs_free char *number_info = NULL;
   guint state;
 
   state = event->state & gtk_accelerator_get_default_mod_mask ();
 
-  matched_string = terminal_screen_check_match (screen, (GdkEvent*)event, &matched_flavor);
+  url = terminal_screen_check_match (screen, (GdkEvent*)event, &url_flavor);
+  terminal_screen_check_extra (screen, (GdkEvent*)event, &number_info);
 
-  if (matched_string != NULL &&
+  if (url != NULL &&
       (event->button == 1 || event->button == 2) &&
       (state & GDK_CONTROL_MASK))
     {
       gboolean handled = FALSE;
 
       g_signal_emit (screen, signals[MATCH_CLICKED], 0,
-                     matched_string,
-                     matched_flavor,
+                     url,
+                     url_flavor,
                      state,
                      &handled);
       if (handled)
@@ -1481,15 +1554,17 @@ terminal_screen_button_press (GtkWidget      *widget,
           if (button_press_event && button_press_event (widget, event))
             return TRUE;
 
-          terminal_screen_do_popup (screen, event, matched_string, matched_flavor);
-          matched_string = NULL; /* adopted to the popup info */
+          terminal_screen_do_popup (screen, event, url, url_flavor, number_info);
+          url = NULL; /* adopted to the popup info */
+          number_info = NULL; /* detto */
           return TRUE;
         }
       else if (!(event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK)))
         {
           /* do popup on shift+right-click */
-          terminal_screen_do_popup (screen, event, matched_string, matched_flavor);
-          matched_string = NULL; /* adopted to the popup info */
+          terminal_screen_do_popup (screen, event, url, url_flavor, number_info);
+          url = NULL; /* adopted to the popup info */
+          number_info = NULL; /* detto */
           return TRUE;
         }
     }
@@ -1852,6 +1927,55 @@ terminal_screen_check_match (TerminalScreen *screen,
   return NULL;
 }
 
+static void
+terminal_screen_check_extra (TerminalScreen *screen,
+                             GdkEvent       *event,
+                             char           **number_info)
+{
+  guint i;
+  char **matches;
+  gboolean flavor_number_found = FALSE;
+
+  matches = g_newa (char *, n_extra_regexes);
+
+  if (
+#ifdef WITH_PCRE2
+      vte_terminal_event_check_regex_simple(
+#else
+      vte_terminal_event_check_gregex_simple(
+#endif
+                                             VTE_TERMINAL (screen),
+                                             event,
+                                             extra_regexes,
+                                             n_extra_regexes,
+                                             0,
+                                             matches))
+    {
+      for (i = 0; i < n_extra_regexes; i++)
+        {
+          if (matches[i] != NULL)
+            {
+              /* Store the first match for each flavor, free all the others */
+              switch (extra_regex_flavors[i])
+                {
+                case FLAVOR_NUMBER:
+                  if (!flavor_number_found)
+                    {
+                      *number_info = terminal_util_number_info (matches[i]);
+                      flavor_number_found = TRUE;
+                    }
+                  g_free (matches[i]);
+                  break;
+                default:
+                  g_free (matches[i]);
+                }
+            }
+        }
+    }
+
+  return NULL;
+}
+
 /**
  * terminal_screen_has_foreground_process:
  * @screen:
